@@ -61,8 +61,8 @@ class VolumeHostAttributeTest(test.TestCase):
 
     def setUp(self):
         super(VolumeHostAttributeTest, self).setUp()
-        self.stubs.Set(volume.API, 'get', fake_volume_get)
-        self.stubs.Set(volume.API, 'get_all', fake_volume_get_all)
+        self.stubs.Set(volume.api.API, 'get', fake_volume_get)
+        self.stubs.Set(volume.api.API, 'get_all', fake_volume_get_all)
         self.stubs.Set(db, 'volume_get', fake_volume_get)
 
         self.UUID = uuid.uuid4()
