@@ -23,6 +23,7 @@ import webob
 from cinder.api import common
 from cinder.api.contrib import volume_image_metadata
 from cinder.api.openstack import wsgi
+from cinder import context
 from cinder import db
 from cinder import exception
 from cinder import test
@@ -51,6 +52,10 @@ def fake_volume_get_all(*args, **kwargs):
     return [fake_volume_get()]
 
 
+def fake_volume_get_all_empty(*args, **kwargs):
+    return []
+
+
 fake_image_metadata = {
     'image_id': 'someid',
     'image_name': 'fake',
@@ -89,11 +94,11 @@ class VolumeImageMetadataTest(test.TestCase):
 
     def setUp(self):
         super(VolumeImageMetadataTest, self).setUp()
-        self.stubs.Set(volume.API, 'get', fake_volume_get)
-        self.stubs.Set(volume.API, 'get_all', fake_volume_get_all)
-        self.stubs.Set(volume.API, 'get_volume_image_metadata',
+        self.stubs.Set(volume.api.API, 'get', fake_volume_get)
+        self.stubs.Set(volume.api.API, 'get_all', fake_volume_get_all)
+        self.stubs.Set(volume.api.API, 'get_volume_image_metadata',
                        fake_get_volume_image_metadata)
-        self.stubs.Set(volume.API, 'get_volumes_image_metadata',
+        self.stubs.Set(volume.api.API, 'get_volumes_image_metadata',
                        fake_get_volumes_image_metadata)
         self.stubs.Set(db, 'volume_get', fake_volume_get)
         self.UUID = uuid.uuid4()
@@ -115,20 +120,58 @@ class VolumeImageMetadataTest(test.TestCase):
             for volume in json.loads(body)['volumes']
         ]
 
+    def _create_volume_and_glance_metadata(self):
+        ctxt = context.get_admin_context()
+        db.volume_create(ctxt, {'id': 'fake', 'status': 'available',
+                                'host': 'test', 'provider_location': '',
+                                'size': 1})
+        db.volume_glance_metadata_create(ctxt, 'fake', 'image_id', 'someid')
+        db.volume_glance_metadata_create(ctxt, 'fake', 'image_name', 'fake')
+        db.volume_glance_metadata_create(ctxt, 'fake', 'kernel_id',
+                                         'somekernel')
+        db.volume_glance_metadata_create(ctxt, 'fake', 'ramdisk_id',
+                                         'someramdisk')
+
     def test_get_volume(self):
+        self._create_volume_and_glance_metadata()
         res = self._make_request('/v2/fake/volumes/%s' % self.UUID)
         self.assertEqual(200, res.status_int)
         self.assertEqual(fake_image_metadata,
                          self._get_image_metadata(res.body))
 
     def test_list_detail_volumes(self):
+        self._create_volume_and_glance_metadata()
         res = self._make_request('/v2/fake/volumes/detail')
         self.assertEqual(200, res.status_int)
         self.assertEqual(fake_image_metadata,
                          self._get_image_metadata_list(res.body)[0])
 
+    def test_list_detail_empty_volumes(self):
+        def fake_dont_call_this(*args, **kwargs):
+            fake_dont_call_this.called = True
+        fake_dont_call_this.called = False
+        self.stubs.Set(volume.api.API, 'get_list_volumes_image_metadata',
+                       fake_dont_call_this)
+        self.stubs.Set(volume.api.API, 'get_all', fake_volume_get_all_empty)
+
+        res = self._make_request('/v2/fake/volumes/detail')
+        self.assertEqual(200, res.status_int)
+        self.assertFalse(fake_dont_call_this.called)
+
+    def test_list_detail_volumes_with_limit(self):
+        ctxt = context.get_admin_context()
+        db.volume_create(ctxt, {'id': 'fake', 'status': 'available',
+                                'host': 'test', 'provider_location': '',
+                                'size': 1})
+        db.volume_glance_metadata_create(ctxt, 'fake', 'key1', 'value1')
+        db.volume_glance_metadata_create(ctxt, 'fake', 'key2', 'value2')
+        res = self._make_request('/v2/fake/volumes/detail?limit=1')
+        self.assertEqual(200, res.status_int)
+        self.assertEqual({'key1': 'value1', 'key2': 'value2'},
+                         self._get_image_metadata_list(res.body)[0])
+
     def test_create_image_metadata(self):
-        self.stubs.Set(volume.API, 'get_volume_image_metadata',
+        self.stubs.Set(volume.api.API, 'get_volume_image_metadata',
                        return_empty_image_metadata)
         self.stubs.Set(db, 'volume_metadata_update',
                        fake_create_volume_metadata)
@@ -147,7 +190,7 @@ class VolumeImageMetadataTest(test.TestCase):
     def test_create_with_keys_case_insensitive(self):
         # If the keys in uppercase_and_lowercase, should return the one
         # which server added
-        self.stubs.Set(volume.API, 'get_volume_image_metadata',
+        self.stubs.Set(volume.api.API, 'get_volume_image_metadata',
                        return_empty_image_metadata)
         self.stubs.Set(db, 'volume_metadata_update',
                        fake_create_volume_metadata)
@@ -182,7 +225,7 @@ class VolumeImageMetadataTest(test.TestCase):
                           self.controller.create, req, 1, None)
 
     def test_create_nonexistent_volume(self):
-        self.stubs.Set(volume.API, 'get', return_volume_nonexistent)
+        self.stubs.Set(volume.api.API, 'get', return_volume_nonexistent)
 
         req = fakes.HTTPRequest.blank('/v2/fake/volumes/1/action')
         req.method = 'POST'
