@@ -26,7 +26,6 @@ from cinder.tests.unit.volume.drivers.netapp.eseries import fakes as \
     eseries_fake
 from cinder.volume.drivers.netapp.eseries import exception as es_exception
 
-
 from cinder.volume.drivers.netapp.eseries import client
 from cinder.volume.drivers.netapp import utils as na_utils
 
@@ -70,18 +69,21 @@ class NetAppEseriesClientDriverTestCase(test.TestCase):
 
             self.assertEqual(status_code, exc.status_code)
 
-    def test_eval_response_422(self):
+    @ddt.data(('30', 'storage array password.*?incorrect'),
+              ('authFailPassword', 'storage array password.*?incorrect'),
+              ('unknown', None))
+    @ddt.unpack
+    def test_eval_response_422(self, ret_code, exc_regex):
         status_code = 422
-        resp_text = "Fake Error Message"
         fake_resp = mock.Mock()
+        fake_resp.text = "fakeError"
+        fake_resp.json = mock.Mock(return_value={'retcode': ret_code})
         fake_resp.status_code = status_code
-        fake_resp.text = resp_text
-        expected_msg = "Response error - %s." % resp_text
+        exc_regex = exc_regex if exc_regex is not None else fake_resp.text
 
-        with self.assertRaisesRegex(es_exception.WebServiceException,
-                                    expected_msg) as exc:
+        with self.assertRaisesRegexp(es_exception.WebServiceException,
+                                     exc_regex) as exc:
             self.my_client._eval_response(fake_resp)
-
             self.assertEqual(status_code, exc.status_code)
 
     def test_register_storage_system_does_not_log_password(self):
@@ -343,16 +345,8 @@ class NetAppEseriesClientDriverTestCase(test.TestCase):
                 eseries_fake.FAKE_ASUP_DATA['operating-mode'],
                 eseries_fake.FAKE_ABOUT_RESPONSE['version'])))
         self.mock_object(
-            self.my_client, 'get_firmware_version',
-            mock.Mock(
-                return_value=eseries_fake.FAKE_ABOUT_RESPONSE['version']))
-        self.mock_object(
-            self.my_client, 'get_serial_numbers',
-            mock.Mock(return_value=eseries_fake.FAKE_SERIAL_NUMBERS))
-        self.mock_object(
-            self.my_client, 'get_model_name',
-            mock.Mock(
-                return_value=eseries_fake.FAKE_CONTROLLERS[0]['modelName']))
+            self.my_client, 'get_asup_info',
+            mock.Mock(return_value=eseries_fake.GET_ASUP_RETURN))
         self.mock_object(
             self.my_client, 'set_counter',
             mock.Mock(return_value={'value': 1}))
@@ -369,38 +363,48 @@ class NetAppEseriesClientDriverTestCase(test.TestCase):
         mock_invoke.assert_called_with(*eseries_fake.FAKE_POST_INVOKE_DATA)
 
     @ddt.data((eseries_fake.FAKE_SERIAL_NUMBERS,
-               eseries_fake.FAKE_CONTROLLERS),
-              (eseries_fake.FAKE_DEFAULT_SERIAL_NUMBER, []),
+               eseries_fake.HARDWARE_INVENTORY),
+              (eseries_fake.FAKE_DEFAULT_SERIAL_NUMBER, {}),
               (eseries_fake.FAKE_SERIAL_NUMBER,
-              eseries_fake.FAKE_SINGLE_CONTROLLER))
+               eseries_fake.HARDWARE_INVENTORY_SINGLE_CONTROLLER))
     @ddt.unpack
-    def test_get_serial_numbers(self, expected_serial_numbers, controllers):
+    def test_get_asup_info_serial_numbers(self, expected_serial_numbers,
+                                          controllers):
         self.mock_object(
-            client.RestClient, '_get_controllers',
+            client.RestClient, 'list_hardware_inventory',
             mock.Mock(return_value=controllers))
-
-        serial_numbers = client.RestClient.get_serial_numbers(self.my_client)
-
-        self.assertEqual(expected_serial_numbers, serial_numbers)
-
-    def test_get_model_name(self):
         self.mock_object(
-            client.RestClient, '_get_controllers',
-            mock.Mock(return_value=eseries_fake.FAKE_CONTROLLERS))
+            client.RestClient, 'list_storage_system',
+            mock.Mock(return_value={}))
 
-        model = client.RestClient.get_model_name(self.my_client)
+        sn = client.RestClient.get_asup_info(self.my_client)['serial_numbers']
 
-        self.assertEqual(eseries_fake.FAKE_CONTROLLERS[0]['modelName'],
-                         model)
+        self.assertEqual(expected_serial_numbers, sn)
 
-    def test_get_model_name_empty_controllers_list(self):
+    def test_get_asup_info_model_name(self):
         self.mock_object(
-            client.RestClient, '_get_controllers',
-            mock.Mock(return_value=[]))
+            client.RestClient, 'list_hardware_inventory',
+            mock.Mock(return_value=eseries_fake.HARDWARE_INVENTORY))
+        self.mock_object(
+            client.RestClient, 'list_storage_system',
+            mock.Mock(return_value=eseries_fake.STORAGE_SYSTEM))
 
-        model = client.RestClient.get_model_name(self.my_client)
+        model_name = client.RestClient.get_asup_info(self.my_client)['model']
 
-        self.assertEqual(eseries_fake.FAKE_DEFAULT_MODEL, model)
+        self.assertEqual(eseries_fake.HARDWARE_INVENTORY['controllers'][0]
+                         ['modelName'], model_name)
+
+    def test_get_asup_info_model_name_empty_controllers_list(self):
+        self.mock_object(
+            client.RestClient, 'list_hardware_inventory',
+            mock.Mock(return_value={}))
+        self.mock_object(
+            client.RestClient, 'list_storage_system',
+            mock.Mock(return_value={}))
+
+        model_name = client.RestClient.get_asup_info(self.my_client)['model']
+
+        self.assertEqual(eseries_fake.FAKE_DEFAULT_MODEL, model_name)
 
     def test_get_eseries_api_info(self):
         fake_invoke_service = mock.Mock()
@@ -677,8 +681,7 @@ class NetAppEseriesClientDriverTestCase(test.TestCase):
 
         self.my_client._invoke.assert_called_once_with('DELETE', url,
                                                        **{'object-id':
-                                                          fake_volume['id']}
-                                                       )
+                                                          fake_volume['id']})
 
     @ddt.data('00.00.00.00', '01.52.9000.2', '01.52.9001.2', '01.51.9000.3',
               '01.51.9001.3', '01.51.9010.5', '0.53.9000.3', '0.53.9001.4')
@@ -733,7 +736,6 @@ class NetAppEseriesClientDriverTestCase(test.TestCase):
 
 @ddt.ddt
 class TestWebserviceClientTestCase(test.TestCase):
-
     def setUp(self):
         """sets up the mock tests"""
         super(TestWebserviceClientTestCase, self).setUp()
