@@ -16,7 +16,7 @@
 
 """Handles all requests relating to volumes."""
 
-
+import ast
 import collections
 import datetime
 import functools
@@ -505,6 +505,7 @@ class API(base.Base):
         return volumes
 
     def get_snapshot(self, context, snapshot_id):
+        check_policy(context, 'get_snapshot')
         snapshot = objects.Snapshot.get_by_id(context, snapshot_id)
 
         # FIXME(jdg): The objects don't have the db name entries
@@ -1154,6 +1155,15 @@ class API(base.Base):
                  resource=volume)
         return {meta_entry.key: meta_entry.value for meta_entry in db_data}
 
+    def get_list_volumes_image_metadata(self, context, volume_id_list):
+        db_data = self.db.volume_glance_metadata_list_get(context,
+                                                          volume_id_list)
+        results = collections.defaultdict(dict)
+        for meta_entry in db_data:
+            results[meta_entry['volume_id']].update({meta_entry['key']:
+                                                     meta_entry['value']})
+        return results
+
     def _check_volume_availability(self, volume, force):
         """Check if the volume can be used."""
         if volume['status'] not in ['available', 'in-use']:
@@ -1365,18 +1375,19 @@ class API(base.Base):
         # This is a volume swap initiated by Nova, not Cinder. Nova expects
         # us to return the new_volume_id.
         if not (volume['migration_status'] or new_volume['migration_status']):
-            # Don't need to do migration, but still need to finish the
-            # volume attach and detach so volumes don't end in 'attaching'
-            # and 'detaching' state
-            attachments = volume['volume_attachment']
-            for attachment in attachments:
-                self.detach(context, volume, attachment['id'])
+            # When we're not migrating and haven't hit any errors, we issue
+            # volume attach and detach requests so the volumes don't end in
+            # 'attaching' and 'detaching' state
+            if not error:
+                attachments = volume['volume_attachment']
+                for attachment in attachments:
+                    self.detach(context, volume, attachment['id'])
 
-                self.attach(context, new_volume,
-                            attachment['instance_uuid'],
-                            attachment['attached_host'],
-                            attachment['mountpoint'],
-                            'rw')
+                    self.attach(context, new_volume,
+                                attachment['instance_uuid'],
+                                attachment['attached_host'],
+                                attachment['mountpoint'],
+                                'rw')
 
             return new_volume['id']
 
@@ -1704,6 +1715,35 @@ class API(base.Base):
         # level to show an admin how a backend is configured.
         return self.volume_rpcapi.list_replication_targets(ctxt, volume)
 
+    def check_volume_filters(self, filters):
+        '''Sets the user filter value to accepted format'''
+        booleans = self.db.get_booleans_for_table('volume')
+
+        # To translate any true/false equivalent to True/False
+        # which is only acceptable format in database queries.
+        accepted_true = ['True', 'true', 'TRUE']
+        accepted_false = ['False', 'false', 'FALSE']
+        for k, v in filters.items():
+            try:
+                if k in booleans:
+                    if v in accepted_false:
+                        filters[k] = False
+                    elif v in accepted_true:
+                        filters[k] = True
+                    else:
+                        filters[k] = bool(v)
+                elif k == 'display_name':
+                    # Use the raw value of display name as is for the filter
+                    # without passing it through ast.literal_eval(). If the
+                    # display name is a properly quoted string (e.g. '"foo"')
+                    # then literal_eval() strips the quotes (i.e. 'foo'), so
+                    # the filter becomes different from the user input.
+                    continue
+                else:
+                    filters[k] = ast.literal_eval(v)
+            except (ValueError, SyntaxError):
+                LOG.debug('Could not evaluate value %s, assuming string', v)
+
 
 class HostAPI(base.Base):
     def __init__(self):
