@@ -88,7 +88,7 @@ BANDWIDTH_LIMIT = 'sio:bandwidth_limit'
 
 BLOCK_SIZE = 8
 OK_STATUS_CODE = 200
-VOLUME_NOT_FOUND_ERROR = 78
+VOLUME_NOT_FOUND_ERROR = 79
 VOLUME_NOT_MAPPED_ERROR = 84
 VOLUME_ALREADY_MAPPED_ERROR = 81
 
@@ -542,10 +542,10 @@ class ScaleIODriver(driver.VolumeDriver):
 
         # Round up the volume size so that it is a granularity of 8 GBs
         # because ScaleIO only supports volumes with a granularity of 8 GBs.
-        if new_size % 8 == 0:
-            volume_new_size = new_size
-        else:
-            volume_new_size = new_size + 8 - (new_size % 8)
+        volume_new_size = self._round_to_8_gran(new_size)
+        volume_real_old_size = self._round_to_8_gran(volume.size)
+        if volume_real_old_size == volume_new_size:
+            return
 
         round_volume_capacity = self.configuration.sio_round_volume_capacity
         if (not round_volume_capacity and not new_size % 8 == 0):
@@ -571,6 +571,11 @@ class ScaleIODriver(driver.VolumeDriver):
             LOG.error(msg)
             raise exception.VolumeBackendAPIException(data=msg)
 
+    def _round_to_8_gran(self, size):
+        if size % 8 == 0:
+            return size
+        return size + 8 - (size % 8)
+
     def create_cloned_volume(self, volume, src_vref):
         """Creates a cloned volume."""
         volume_id = src_vref['provider_id']
@@ -636,7 +641,7 @@ class ScaleIODriver(driver.VolumeDriver):
         if r.status_code != OK_STATUS_CODE:
             response = r.json()
             error_code = response['errorCode']
-            if error_code == 78:
+            if error_code == VOLUME_NOT_FOUND_ERROR:
                 force_delete = self.configuration.sio_force_delete
                 if force_delete:
                     LOG.warning(_LW(
