@@ -114,6 +114,7 @@ class NetAppESeriesLibrary(object):
     RAID_UQ_SPEC = 'netapp_raid_type'
     THIN_UQ_SPEC = 'netapp_thin_provisioned'
     SSC_UPDATE_INTERVAL = 60  # seconds
+    SA_COMM_TIMEOUT = 30
     WORLDWIDENAME = 'worldWideName'
 
     DEFAULT_HOST_TYPE = 'linux_dm_mp'
@@ -247,48 +248,104 @@ class NetAppESeriesLibrary(object):
         self._client.set_system_id(system.get('id'))
         self._client._init_features()
 
+    def _check_password_status(self, system):
+        """Determine if the storage system's password status is valid.
+
+        The password status has the following possible states: unknown, valid,
+        invalid.
+
+        If the password state cannot be retrieved from the storage system,
+        an empty string will be returned as the status, and the password
+        status will be assumed to be valid. This is done to ensure that
+        access to a storage system will not be blocked in the event of a
+        problem with the API.
+
+        This method returns a tuple consisting of the storage system's
+        password status and whether or not the status is valid.
+
+        Example: (invalid, True)
+
+        :returns (str, bool)
+        """
+
+        status = system.get('passwordStatus')
+        status = status.lower() if status else ''
+        return status, status not in ['invalid', 'unknown']
+
+    def _check_storage_system_status(self, system):
+        """Determine if the storage system's status is valid.
+
+        The storage system status has the following possible states:
+        neverContacted, offline, optimal, needsAttn.
+
+        If the storage system state cannot be retrieved, an empty string will
+        be returned as the status, and the storage system's status will be
+        assumed to be valid. This is done to ensure that access to a storage
+        system will not be blocked in the event of a problem with the API.
+
+        This method returns a tuple consisting of the storage system's
+        password status and whether or not the status is valid.
+
+        Example: (needsAttn, True)
+
+        :returns (str, bool)
+        """
+        status = system.get('status')
+        status = status.lower() if status else ''
+        return status, status not in ['nevercontacted', 'offline']
+
     def _check_storage_system(self):
         """Checks whether system is registered and has good status."""
         try:
-            system = self._client.list_storage_system()
+            self._client.list_storage_system()
         except exception.NetAppDriverException:
             with excutils.save_and_reraise_exception():
                 LOG.info(_LI("System with controller addresses [%s] is not "
                              "registered with web service."),
                          self.configuration.netapp_controller_ips)
-        password_not_in_sync = False
-        if system.get('status', '').lower() == 'passwordoutofsync':
-            password_not_in_sync = True
-            new_pwd = self.configuration.netapp_sa_password
-            self._client.update_stored_system_password(new_pwd)
-            time.sleep(self.SLEEP_SECS)
-        sa_comm_timeout = 60
-        comm_time = 0
-        while True:
+
+        # Update the stored password
+        # We do this to trigger the webservices password validation routine
+        new_pwd = self.configuration.netapp_sa_password
+        self._client.update_stored_system_password(new_pwd)
+
+        start_time = int(time.time())
+
+        def check_system_status():
             system = self._client.list_storage_system()
-            status = system.get('status', '').lower()
+            pass_status, pass_status_valid = (
+                self._check_password_status(system))
+            status, status_valid = self._check_storage_system_status(system)
+            msg_dict = {'id': system.get('id'), 'status': status,
+                        'pass_status': pass_status}
             # wait if array not contacted or
             # password was not in sync previously.
-            if ((status == 'nevercontacted') or
-                    (password_not_in_sync and status == 'passwordoutofsync')):
-                LOG.info(_LI('Waiting for web service array communication.'))
-                time.sleep(self.SLEEP_SECS)
-                comm_time = comm_time + self.SLEEP_SECS
-                if comm_time >= sa_comm_timeout:
-                    msg = _("Failure in communication between web service and"
-                            " array. Waited %s seconds. Verify array"
-                            " configuration parameters.")
-                    raise exception.NetAppDriverException(msg %
-                                                          sa_comm_timeout)
+            if not (pass_status_valid and status_valid):
+                if not pass_status_valid:
+                    LOG.info(_LI('Waiting for web service to validate the '
+                                 'configured password.'))
+                else:
+                    LOG.info(_LI('Waiting for web service array '
+                                 'communication.'))
+                if int(time.time() - start_time) >= self.SA_COMM_TIMEOUT:
+                    if not status_valid:
+                        raise exception.NetAppDriverException(
+                            _("System %(id)s found with bad status - "
+                              "%(status)s.") % msg_dict)
+                    else:
+                        raise exception.NetAppDriverException(
+                            _("System %(id)s found with bad password status - "
+                              "%(pass_status)s.") % msg_dict)
+
+            # The system was found to have a good status
             else:
-                break
-        msg_dict = {'id': system.get('id'), 'status': status}
-        if (status == 'passwordoutofsync' or status == 'notsupported' or
-                status == 'offline'):
-            raise exception.NetAppDriverException(
-                _("System %(id)s found with bad status - "
-                  "%(status)s.") % msg_dict)
-        LOG.info(_LI("System %(id)s has %(status)s status."), msg_dict)
+                LOG.info(_LI("System %(id)s has %(status)s status."), msg_dict)
+                raise loopingcall.LoopingCallDone()
+
+        checker = loopingcall.FixedIntervalLoopingCall(f=check_system_status)
+        checker.start(interval = self.SLEEP_SECS,
+                      initial_delay=self.SLEEP_SECS).wait()
+
         return True
 
     def _get_volume(self, uid):
@@ -968,12 +1025,14 @@ class NetAppESeriesLibrary(object):
             LOG.info(msg % self._client.api_version)
             return
 
-        firmware_version = self._client.get_firmware_version()
         event_source = ("Cinder driver %s" % self.DRIVER_NAME)
         category = "provisioning"
         event_description = "OpenStack Cinder connected to E-Series proxy"
-        model = self._client.get_model_name()
-        serial_numbers = self._client.get_serial_numbers()
+        asup_info = self._client.get_asup_info()
+        model = asup_info.get('model')
+        firmware_version = asup_info.get('firmware_version')
+        serial_numbers = asup_info.get('serial_numbers')
+        chassis_sn = asup_info.get('chassis_sn')
 
         key = ("openstack-%s-%s-%s"
                % (cinder_host, serial_numbers[0], serial_numbers[1]))
@@ -991,6 +1050,7 @@ class NetAppESeriesLibrary(object):
             'event-description': event_description,
             'controller1-serial': serial_numbers[0],
             'controller2-serial': serial_numbers[1],
+            'chassis-serial-number': chassis_sn,
             'model': model,
             'system-version': firmware_version,
             'operating-mode': self._client.api_operating_mode
